package com.kakao.sdk.auth

import com.kakao.sdk.auth.model.OAuthToken
import com.kakao.sdk.auth.model.AccessTokenResponse

/**
 * 카카오 API 에 사용되는 액세스 토큰, 리프레시 토큰을 관리하는 저장소.
 *
 * Repository for access and refresh token used for Kakao API.
 *
 * @since 2.0.0
 *
 * @author kevin.kang. Created on 2018. 3. 24..
 */
interface AccessTokenRepo {
    /**
     * 캐시에 저장되어 있는 [OAuthToken] 객체를 리턴.
     *
     * @return OAuthToken instance stored in the cache
     */
    fun fromCache(): OAuthToken

    /**
     * OAuth Token API 를 통하여 받은 [AccessTokenResponse] 객체를 사용하여 캐시를 업데이트 함.
     *
     * @param accessTokenResponse [AccessTokenResponse] from [RxAuthApiClient.issueAccessToken] API
     * @return updated [OAuthToken] instance
     */
    fun toCache(accessTokenResponse: AccessTokenResponse): OAuthToken

    /**
     * 캐시에 저장되어 있는 [OAuthToken] 객체를 지움.
     * Clear [OAuthToken] in cache used by the SDK
     */
    fun clearCache()

    companion object {
        @JvmStatic
        val instance: AccessTokenRepo by lazy {
            DefaultAccessTokenRepo()
        }
    }
}