package com.kakao.sdk.link.model

import com.google.gson.JsonObject
import com.kakao.sdk.common.KakaoJson
import com.kakao.sdk.common.util.Utility
import com.kakao.sdk.link.Constants
import org.junit.jupiter.api.Assertions.assertEquals
import org.junit.jupiter.params.ParameterizedTest
import org.junit.jupiter.params.provider.Arguments
import org.junit.jupiter.params.provider.MethodSource
import java.util.stream.Stream

/**
 * @author kevin.kang. Created on 2018. 4. 30..
 */
class ValidationResultTest {
    @MethodSource("responseProvider")
    @ParameterizedTest
    fun parse(jsonString: String, jsonObject: JsonObject) {
        val response =
            KakaoJson.fromJson<ValidationResult>(jsonString, ValidationResult::class.java)
        assertEquals(jsonObject[Constants.TEMPLATE_ID].asLong, response.templateId)
    }

    companion object {
        val paths = listOf(
            "default_commerce", "default_feed", "default_list", "default_location", "default_text",
            "validate"
        )

        @Suppress("unused")
        @JvmStatic
        fun responseProvider(): Stream<Arguments> {
            return paths
                .map { "json/$it.json" }
                .map { Utility.getJson(it) }
                .map { it to KakaoJson.fromJson<JsonObject>(it, JsonObject::class.java) }
                .map { Arguments.of(it.first, it.second) }
                .stream()
        }
    }

}