package com.kakao.sdk.network

import com.kakao.sdk.common.*
import okhttp3.OkHttpClient
import okhttp3.logging.HttpLoggingInterceptor
import retrofit2.Retrofit
import retrofit2.adapter.rxjava2.RxJava2CallAdapterFactory
import retrofit2.converter.gson.GsonConverterFactory

/**
 * @author kevin.kang. Created on 2018. 3. 20..
 */

fun ApiFactory.withClient(url: String, clientBuilder: OkHttpClient.Builder): Retrofit {
    return Retrofit.Builder().baseUrl(url)
        .addCallAdapterFactory(RxJava2CallAdapterFactory.create())
        .addConverterFactory(KakaoRetrofitConverterFactory())
        .addConverterFactory(GsonConverterFactory.create(KakaoJson.base))
        .client(clientBuilder.build())
        .build()
}

val ApiFactory.kapi by lazy {
    ApiFactory.withClientAndAdapter(
        "${Constants.SCHEME}://${KakaoSdk.serverHosts.kapi}",
        OkHttpClient.Builder()
            .addInterceptor(KakaoAgentInterceptor())
            .addInterceptor(AppKeyInterceptor())
            .addInterceptor(ApiFactory.loggingInterceptor),
        RxJava2CallAdapterFactory.create()
    )
}

val ApiFactory.loggingInterceptor by lazy {
    val interceptor = HttpLoggingInterceptor(object : HttpLoggingInterceptor.Logger {
        override fun log(message: String) {
            SdkLogger.rx.i(message)
        }
    })

    interceptor.level =
        if (BuildConfig.DEBUG) HttpLoggingInterceptor.Level.BODY else HttpLoggingInterceptor.Level.HEADERS
    return@lazy interceptor
}