package com.kakao.sdk.common

import com.kakao.sdk.common.json.IntDate
import com.kakao.sdk.common.json.MapToQuery
import com.kakao.sdk.common.util.Utility
import retrofit2.Converter
import retrofit2.Retrofit
import java.lang.reflect.ParameterizedType
import java.lang.reflect.Type
import java.util.*

/**
 * @suppress
 * @author kevin.kang. Created on 2018. 3. 21..
 */
class KakaoRetrofitConverterFactory : Converter.Factory() {
    override fun stringConverter(
        type: Type?,
        annotations: Array<out Annotation>?,
        retrofit: Retrofit?
    ): Converter<*, String>? {
        if (type == String::class.java) {
            return null
        }
        if (type is Class<*> && type.isEnum) {
            return Converter { enum: Enum<*> ->
                val encoded = KakaoJson.toJson(enum)
                return@Converter encoded.substring(1, encoded.length - 1)
            }
        }
        if (type == Date::class.java) {
            annotations?.filterIsInstance<IntDate>()?.firstOrNull()?.let {
                return Converter { value: Date -> (value.time / 1000).toString() }
            }
        }
        if (type is ParameterizedType && type.rawType == Map::class.java) {
            annotations?.filterIsInstance<MapToQuery>()?.firstOrNull()?.let {
                return Converter { map: Map<String, String> -> Utility.buildQuery(map) }
            }
        }
        return Converter { value: Any ->
            KakaoJson.toJson(value)
        }
    }
}