package com.kakao.sdk.story.model

import android.os.Parcelable
import com.google.gson.annotations.SerializedName
import com.kakao.sdk.common.json.UnknownValue
import kotlinx.android.parcel.Parcelize
import java.util.*

/**
 * @author kevin.kang. Created on 2018. 3. 20..
 */
@Parcelize
data class Story(
    val id: String,
    val url: String,
    val mediaType: String,
    val createdAt: Date,
    val commentCount: Int,
    val likeCount: Int,
    val content: String,
    val permission: String,
    val media: List<StoryImage>?,
    val likes: List<StoryLike>,
    val comments: List<StoryComment>
) : Parcelable {
    enum class Permission {
        /**
         * 전체공개
         */
        @SerializedName("A")
        PUBLIC,
        /**
         * 친구공개
         */
        @SerializedName("F")
        FRIEND,
        /**
         * 나만보기
         */
        @SerializedName("M")
        ONLY_ME,

        @UnknownValue
        UNKNOWN;
    }
}

@Parcelize
data class StoryImage(
    val xlarge: String,
    val large: String,
    val medium: String,
    val small: String,
    val original: String
) : Parcelable

@Parcelize
data class StoryActor(val displayName: String, val profileThumbnailUrl: String) : Parcelable

@Parcelize
data class StoryComment(val writer: StoryActor, val text: String) : Parcelable

@Parcelize
data class StoryLike(val actor: StoryActor, val emotion: Emotion) : Parcelable {
    enum class Emotion {
        /**
         * 좋아요
         */
        @SerializedName("LIKE")
        LIKE,
        /**
         * 멋져요
         */
        @SerializedName("COOL")
        COOL,
        /**
         * 기뻐요
         */
        @SerializedName("HAPPY")
        HAPPY,
        /**
         * 슬퍼요
         */
        @SerializedName("SAD")
        SAD,
        /**
         * 힘내요
         */
        @SerializedName("CHEER_UP")
        CHEER_UP,
        /**
         * 정의되지 않은 느낌
         */
        @UnknownValue
        UNKNOWN;
    }
}