@file:JvmName("UserApiClientKt")

package com.kakao.sdk.user

import com.kakao.sdk.auth.AccessTokenRepo
import com.kakao.sdk.auth.network.TokenBasedApiInterceptor
import com.kakao.sdk.auth.network.kapiWithOAuth
import com.kakao.sdk.common.ApiFactory
import com.kakao.sdk.network.RxApiInterceptor
import com.kakao.sdk.user.model.AccessTokenInfo
import com.kakao.sdk.user.model.User
import com.kakao.sdk.user.model.UserServiceTerms
import com.kakao.sdk.user.model.UserShippingAddresses
import io.reactivex.Completable
import io.reactivex.Single
import java.util.*

/**
 * @author kevin.kang. Created on 2018. 4. 2..
 */
class RxUserApiClient(
    private val userApi: RxUserApi = ApiFactory.kapiWithOAuth.create(RxUserApi::class.java),
    val apiInterceptor: TokenBasedApiInterceptor = TokenBasedApiInterceptor.instance,
    private val accessTokenRepo: AccessTokenRepo = AccessTokenRepo.instance
) {

    @JvmOverloads
    fun me(secureReSource: Boolean = true): Single<User> {
        return userApi.me(secureReSource)
            .compose(RxApiInterceptor.handleApiError())
            .compose(apiInterceptor.handleApiError())
    }

    fun updateProfile(properties: Map<String, String>): Completable {
        return userApi.updateProfile(properties)
            .compose(RxApiInterceptor.handleCompletableError())
            .compose(apiInterceptor.handleCompletableError())
    }

    fun accessTokenInfo(): Single<AccessTokenInfo> {
        return userApi.accessTokenInfo()
            .compose(RxApiInterceptor.handleApiError())
            .compose(apiInterceptor.handleApiError())
    }

    fun logout(): Completable {
        return userApi.logout()
            .compose(RxApiInterceptor.handleCompletableError())
            .compose(apiInterceptor.handleCompletableError())
            .doOnEvent {
                accessTokenRepo.clearCache()
            }
    }

    fun unlink(): Completable {
        return userApi.unlink()
            .compose(RxApiInterceptor.handleCompletableError())
            .compose(apiInterceptor.handleCompletableError())
            .doOnEvent { accessTokenRepo.clearCache() }
    }

    @JvmOverloads
    fun shippingAddresses(
        fromUpdateAt: Date? = null,
        pageSize: Int? = null
    ): Single<UserShippingAddresses> {
        return userApi.shippingAddresses(fromUpdateAt = fromUpdateAt, pageSize = pageSize)
            .compose(RxApiInterceptor.handleApiError())
            .compose(apiInterceptor.handleApiError())
    }

    fun shippingAddresses(addressId: Long): Single<UserShippingAddresses> {
        return userApi.shippingAddresses(addressId)
            .compose(RxApiInterceptor.handleApiError())
            .compose(apiInterceptor.handleApiError())
    }

    fun serviceTerms(): Single<UserServiceTerms> {
        return userApi.serviceTerms()
            .compose(RxApiInterceptor.handleApiError())
            .compose(apiInterceptor.handleApiError())
    }

    companion object {
        @JvmStatic
        val instance by lazy { UserApiClient.rx }
    }
}

val UserApiClient.Companion.rx by lazy { RxUserApiClient() }