package com.kakao.sdk.navi

import android.content.Context
import android.content.Intent
import android.net.Uri
import com.kakao.sdk.common.*
import com.kakao.sdk.navi.model.KakaoNaviParams
import com.kakao.sdk.common.KakaoJson
import com.kakao.sdk.common.model.ApplicationInfo
import com.kakao.sdk.common.model.ContextInfo
import com.kakao.sdk.navi.model.Location
import com.kakao.sdk.navi.model.NaviOptions

/**
 * @author kevin.kang. Created on 18/02/2019..
 */
class KakaoNaviClient(
    private val applicationInfo: ApplicationInfo = KakaoSdk.applicationContextInfo,
    private val contextInfo: ContextInfo = KakaoSdk.applicationContextInfo
) {

    /**
     * 카카오내비 앱 설치 여부.
     *
     * @return true if installed, false otherwise.
     */
    fun isKakaoNaviInstalled(context: Context): Boolean {
        val navi = context.packageManager.getLaunchIntentForPackage(Constants.NAVI_PACKAGE) != null
        val lgNavi =
            context.packageManager.getLaunchIntentForPackage(Constants.LG_NAVI_PACKAGE) != null
        return navi || lgNavi
    }

    @JvmOverloads
    fun navigateUri(
        destination: Location,
        option: NaviOptions? = null,
        viaList: List<Location>? = null
    ): Uri =
        baseUriBuilder(
            KakaoNaviParams(
                destination,
                option,
                viaList
            )
        ).path("${Constants.NAVIGATE}.html").build()

    @JvmOverloads
    fun shareDestinationIntent(
        destination: Location,
        options: NaviOptions? = null,
        viaList: List<Location>? = null
    ): Intent {
        val uri = baseUriBuilder(
            KakaoNaviParams(
                destination,
                options,
                viaList
            )
        ).scheme(Constants.NAVI_SCHEME)
            .authority(Constants.SHARE_POI).build()
        return Intent(Intent.ACTION_VIEW, uri)
    }

    @JvmOverloads
    fun navigateIntent(
        destination: Location,
        option: NaviOptions? = null,
        viaList: List<Location>? = null
    ): Intent {
        val uri = baseUriBuilder(
            KakaoNaviParams(
                destination,
                option,
                viaList
            )
        ).scheme(Constants.NAVI_SCHEME)
            .authority(Constants.NAVIGATE).build()
        return Intent(Intent.ACTION_VIEW, uri)
    }

    private fun baseUriBuilder(
        params: KakaoNaviParams
    ): Uri.Builder = Uri.Builder()
        .scheme(Constants.NAVI_WEB_SCHEME)
        .authority(KakaoSdk.serverHosts.navi)
        .appendQueryParameter(Constants.PARAM, KakaoJson.toJson(params))
        .appendQueryParameter(Constants.APIVER, Constants.APIVER_10)
        .appendQueryParameter(Constants.APPKEY, applicationInfo.clientId)
        .appendQueryParameter(com.kakao.sdk.common.Constants.EXTRAS, contextInfo.extras.toString())

    companion object {
        @JvmStatic
        val instance by lazy { KakaoNaviClient() }
    }
}