@file:JvmName("LinkClientKt")

package com.kakao.sdk.link

import android.content.Context
import android.content.Intent
import com.kakao.sdk.common.ApiFactory
import com.kakao.sdk.template.DefaultTemplate
import com.kakao.sdk.network.RxApiInterceptor
import com.kakao.sdk.network.kapi
import io.reactivex.Single

/**
 * @author kevin.kang. Created on 19/02/2019..
 */
class RxLinkClient(
    private val api: RxLinkApi = ApiFactory.kapi.create(RxLinkApi::class.java),
    private val linkIntentClient: KakaoLinkIntentClient = KakaoLinkIntentClient.instance
) {

    fun isKakaoLinkAvailable(context: Context): Boolean {
        return linkIntentClient.isKakaoLinkAvailable(context)
    }

    fun customTemplate(
        context: Context,
        templateId: Long,
        templateArgs: Map<String, String>? = null,
        serverCallbackArgs: Map<String, String>? = null
    ): Single<Intent> {
        return api.validateCustom(templateId, templateArgs)
            .compose(RxApiInterceptor.handleApiError())
            .map { linkIntentClient.linkIntentFromResponse(context, it, serverCallbackArgs) }
    }

    fun defaultTemplate(
        context: Context,
        defaultTemplate: DefaultTemplate,
        serverCallbackArgs: Map<String, String>? = null
    ): Single<Intent> {
        return api.validateDefault(defaultTemplate)
            .compose(RxApiInterceptor.handleApiError())
            .map { linkIntentClient.linkIntentFromResponse(context, it, serverCallbackArgs) }
    }

    fun scrapTemplate(
        context: Context,
        url: String,
        templateId: Long? = null,
        templateArgs: Map<String, String>? = null,
        serverCallbackArgs: Map<String, String>? = null
    ): Single<Intent> {
        return api.validateScrap(url, templateId, templateArgs)
            .compose(RxApiInterceptor.handleApiError())
            .map { linkIntentClient.linkIntentFromResponse(context, it, serverCallbackArgs) }
    }

    companion object {
        @JvmStatic
        val instance by lazy { RxLinkClient() }
    }
}

val LinkClient.Companion.rx by lazy { RxLinkClient() }