package com.kakao.sdk.network

import com.kakao.sdk.common.model.ApiError
import com.kakao.sdk.common.model.ApiErrorCause
import com.kakao.sdk.common.KakaoJson
import com.kakao.sdk.common.SdkLogger
import com.kakao.sdk.common.model.ApiErrorResponse
import io.reactivex.Completable
import io.reactivex.CompletableTransformer
import io.reactivex.Single
import io.reactivex.SingleTransformer
import retrofit2.HttpException

/**
 * @author kevin.kang. Created on 2019-10-17..
 */
object RxApiInterceptor {

    fun <T> handleApiError(): SingleTransformer<T, T> {
        return SingleTransformer { it ->
            it.onErrorResumeNext { Single.error(translateError(it)) }
                .doOnError { SdkLogger.rx.e(it) }
                .doOnSuccess { SdkLogger.rx.i(it.toString()) }
        }
    }

    fun handleCompletableError(): CompletableTransformer {
        return CompletableTransformer {
            it.onErrorResumeNext {
                Completable.error(
                    translateError(
                        it
                    )
                )
            }
                .doOnError { SdkLogger.rx.e(it) }
                .doOnComplete { }
        }
    }

    @JvmSynthetic
    internal fun translateError(t: Throwable): Throwable {
        try {
            if (t is HttpException) {
                val errorString = t.response()?.errorBody()?.string()
                val response =
                    KakaoJson.fromJson<ApiErrorResponse>(
                        errorString!!,
                        ApiErrorResponse::class.java
                    )
                val cause =
                    KakaoJson.fromJson(
                        response.code.toString(),
                        ApiErrorCause::class.java
                    )
                        ?: ApiErrorCause.Unknown
                return ApiError(t.code(), cause, response)
            }
            return t
        } catch (unexpected: Throwable) {
            return unexpected
        }
    }
}