package com.kakao.sdk.template

import org.junit.jupiter.api.Assertions.*
import org.junit.jupiter.api.Test

/**
example:

```
{
"object_type": "feed",
"
}
```
 */
class FeedTemplateTest {
    @Test
    fun objectTypeExists() {
        val template = FeedTemplate(Content("title", "imageUrl", Link()))
        assertEquals(Constants.TYPE_FEED, template.objectType)
    }

    @Test
    fun social() {
        val url = "https://developers.kakao.com"
        val templateObject = """
{
  "object_type": "feed",
  "url":"$url"
}
            }
        """.trimIndent()

        val template = FeedTemplate(
            content = Content(
                "title", "imageUrl", Link(
                    webUrl = "", mobileWebUrl = ""
                )
            ),
            social = Social(likeCount = 1)
        )
        assertNotNull(template.social)
        assertEquals(1, template.social?.likeCount)
    }

    @Test
    fun addButtons() {
        val template = FeedTemplate(
            content = Content("title", "imageUrl", Link()),
            social = Social(likeCount = 1),
            buttons = listOf(
                Button("title", Link()),
                Button("title", Link())
            )
        )
        assertEquals(2, template.buttons.size)
    }
}